@@ -21,6 +21,11 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef DEBUG_LEASES
+#include <arpa/inet.h>
+#include <stdio.h>
+#endif
+
 #include "die.h"
 #include "leases.h"
 
@@ -70,7 +75,7 @@ void remove_lease(const int i) {
 
 /* function that returns the index of a lease pointer */
 int get_index_by_pointer(const lease * a) {
-	int i = (a - leases) / sizeof(*leases);
+	int i = a - leases;
 	if (i >= 0 && i <= lease_c) return i;
 	else {
 		die("get_index_by_pointer: invalid pointer");
@@ -139,3 +144,26 @@ void do_update_expires() {
 		if (leases[i].expires[2] < next_lease_expires) next_lease_expires = leases[i].expires[2];
 	}
 }
+
+#ifdef DEBUG_LEASES
+/* function that prints a lease */
+void print_lease(int i) {
+	struct in_addr client = { leases[i].client };
+	printf("Client: %s, UDP-Expires: %u, TCP-Expires: %u, Private: %hu, Public: %hu\n",
+			inet_ntoa(client),
+			leases[i].expires[1],
+			leases[i].expires[2],
+			ntohs(leases[i].private_port),
+			ntohs(leases[i].public_port));
+}
+
+/* function to print all leases */
+void print_leases() {
+	printf("----LEASES----\n");
+	int i;
+	for (i=0; i<lease_c; i++) {
+		print_lease(i);
+	}
+	printf("--------------\n");
+}
+#endif
