@@ -17,7 +17,7 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define PROGRAM_VERSION "0.2.1"
+#define PROGRAM_VERSION "0.2.2"
 
 #include <arpa/inet.h>
 //#include <sys/types.h>
@@ -40,6 +40,7 @@
 #include "dnat_api.h"
 #include "natpmp_defs.h"
 
+
 #define ADDRESS_CHECK_INTERVAL 1 /* s */
 
 /* level of verbosity */
@@ -197,7 +198,7 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 
 		if (ntohl(answer_packet.mapping.lifetime) > max_lifetime) {
 			/* lifetime too high, downgrade */
-			if (debuglevel >= 2) printf("Requested lifetime was %u, downgraded to %u\n", ntohl(answer_packet.mapping.lifetime), max_lifetime);
+			debug_printf("Requested lifetime was %u, downgraded to %u\n", ntohl(answer_packet.mapping.lifetime), max_lifetime);
 			answer_packet.mapping.lifetime = htonl(max_lifetime);
 		}
 
@@ -211,7 +212,7 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 				else if (a->expires[(int) protocol] <= next_lease_expires) update_expires = 1;
 				a->expires[(int) protocol] = new_expires;
 				answer_packet.mapping.public_port = a->public_port;
-				if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s updated\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+				debug_printf("Lease with public %s port %hu for client %s updated\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
 			}
 			else {
 				/* lease for other protocol exists, but not for this one*/
@@ -223,7 +224,7 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 				else if (b == 1) {
 					/* manual mapping exists, answer with public port */
 					answer_packet.mapping.public_port = public_port;
-					if (debuglevel >= 2) printf("Manual mapping for public %s port %hu for client %s exists\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+					debug_printf("Manual mapping for public %s port %hu for client %s exists\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
 				}
 				else {
 					/* add the lease to the other one */
@@ -240,7 +241,7 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 								answer_packet.mapping.private_port);
 						if (c == -1) die("create_dnat_rule returned with error");
 					}
-					if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s added\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+					debug_printf("Lease with public %s port %hu for client %s added\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
 				}
 			}
 		}
@@ -252,7 +253,7 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 			else if (b == 1) {
 				/* manual mapping exists, answer with public port */
 				answer_packet.mapping.public_port = public_port;
-				if (debuglevel >= 2) printf("Manual mapping for public %s port %hu for client %s exists\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+				debug_printf("Manual mapping for public %s port %hu for client %s exists\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
 			}
 			else {
 				/* no lease and no manual mapping exist, find a valid port and create a lease */
@@ -276,7 +277,7 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 						answer_packet.mapping.public_port = request_packet->mapping.public_port;
 						answer_packet.mapping.lifetime = request_packet->mapping.lifetime;
 						answer_packet.answer.result = NATPMP_OUTOFRESOURCES;
-						if (debuglevel >= 2) printf("No free ports available\n");
+						debug_printf("No free ports available\n");
 						break;
 					}
 					if (get_lease_by_port(answer_packet.mapping.public_port) == NULL &&
@@ -308,7 +309,7 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 							if (c == -1) die("create_dnat_rule returned with error");
 						}
 
-						if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s created\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+						debug_printf("Lease with public %s port %hu for client %s created\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
 						break;
 					}
 
@@ -330,13 +331,13 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 		if (answer_packet.mapping.public_port == 0 && answer_packet.mapping.private_port == 0) {
 			/* removing all mappings of client (but only for requested protocol) */
 			remove_all = 1;
-			if (debuglevel >= 2) printf("Trying to remove all %s leases for client %s\n", proto(protocol), inet_ntoa(t_addr->sin_addr));
+			debug_printf("Trying to remove all %s leases for client %s\n", proto(protocol), inet_ntoa(t_addr->sin_addr));
 		}
 		else {
 			/* only removing a single mapping */
 			remove_all = 0;
 			a = get_lease_by_client_port(client, answer_packet.mapping.private_port);
-			if (debuglevel >= 2) printf("Trying to remove lease with public %s port %hu for client %s\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
+			debug_printf("Trying to remove lease with public %s port %hu for client %s\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
 		}
 
 		while (remove_all == 0 || (a = get_next_lease_by_client(client, NULL)) != NULL) {
@@ -347,7 +348,7 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 				else if (b == 1) {
 					/* mapping may not be destroyed, it's a manual mapping, answer with refused */
 					answer_packet.answer.result = NATPMP_REFUSED;
-					if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s could not be removed\n", proto(protocol), ntohs(a->public_port), inet_ntoa(t_addr->sin_addr));
+					debug_printf("Lease with public %s port %hu for client %s could not be removed\n", proto(protocol), ntohs(a->public_port), inet_ntoa(t_addr->sin_addr));
 				}
 				else
 				{
@@ -365,12 +366,15 @@ void handle_map_request(const int ufd, const struct sockaddr_in * t_addr, const
 			else {
 				/* lease not found, check for manual mapping */
 				int b = get_dnat_rule_by_client_port(protocol, NULL, client, answer_packet.mapping.private_port);
-				if (b == -1) die("get_dnat_rule_by_client_port returned with error");
+				if (b == -1) {
+					die("get_dnat_rule_by_client_port returned with error");
+				}
 				else if (b == 1) {
 					/* manual mapping found, answer with refused */
 					answer_packet.answer.result = NATPMP_REFUSED;
 					if (debuglevel >= 2) printf("Lease with public %s port %hu for client %s is mapped manually\n", proto(protocol), ntohs(answer_packet.mapping.public_port), inet_ntoa(t_addr->sin_addr));
 				}
+				break;
 			}
 		}
 
@@ -471,19 +475,23 @@ void read_from_socket(const int s_i) {
 	if (packet_request.dummy.header.version != NATPMP_VERSION) {
 		/* Apple's Airport stations send an opcode of 0 here, but it's not defined in the draft,
 		 * perhaps this could be used for fingerprinting my implementation :-) */
+		debug_printf("Handling unsupported request...\n");
 		handle_unsupported_request(ufd_v[s_i].fd, &t_addr, &packet_request.dummy, NATPMP_UNSUPPORTEDVERSION);
 		return;
 	}
 	if (packet_request.dummy.header.op & NATPMP_ANSFLAG) return;
 
 	/* do things depending on the packet's op code */
+	debug_printf("Handling request...\n");
 	switch (packet_request.dummy.header.op) {
 		case NATPMP_PUBLICIPADDRESS :
+			debug_printf("Handling public address...\n");
 			if (pkgsize < (ssize_t) sizeof(natpmp_packet_publicipaddress_request)) return; /* TODO: errorlog */
 			send_publicipaddress(ufd_v[s_i].fd, &t_addr);
 			break;
 		case NATPMP_MAP_UDP :
 		case NATPMP_MAP_TCP :
+			debug_printf("Handling port mapping...\n");
 			if (pkgsize < (ssize_t) sizeof(natpmp_packet_map_request)) return; /* TODO: errorlog */
 			handle_map_request(ufd_v[s_i].fd, &t_addr, &packet_request.map);
 			break;
@@ -499,17 +507,19 @@ void update_time() {
 	unow = a.tv_usec + 1000000 * now;
 }
 
-//__attribute__ ((noreturn))
 void print_usage(const char * program_name) {
 	fprintf(stderr, "Usage: %s [-v | -q] [-b [-p pidfile]] -i public-interface -a private-address ... [-t max-lifetime] [-l lower-port] [-u upper-port] [[--] backend-options]\n", program_name);
-	exit(EXIT_FAILURE);
 }
 
-void do_version() {
+void print_help(const char * program_name) {
+	print_usage(program_name);
+}
+
+void print_version() {
 	printf(
 			"natpmp version " PROGRAM_VERSION ".\n\n"
 
-			"Copyright (C) 2007  Adrian Friedli.\n\n"
+			"Copyright (C) 2007-2008  Adrian Friedli.\n\n"
 
 			"This program is free software; you can redistribute it and/or modify\n"
 			"it under the terms of the GNU General Public License as published by\n"
@@ -544,7 +554,12 @@ void init(int argc, char * argv[]) {
 	port_range_low = 1024; /* ports below 1024 are restricted ports */
 	port_range_high = 65535; /* 65535 is the highest port available */
 
-#define OPTSTRING "Vvqbp:i:a:t:l:u:"
+	if (argc==1) {
+		print_usage(argv[0]);
+		exit(EXIT_FAILURE);
+	}
+
+#define OPTSTRING "hVvqbp:i:a:t:l:u:"
 	/* parse the command line */
 	{
 		extern char *optarg;
@@ -556,13 +571,15 @@ void init(int argc, char * argv[]) {
 		opterr = 0;
 		while ( (opt = getopt(argc, argv, OPTSTRING)) != -1 ) {
 			switch (opt) {
+				case 'h':
+					print_help(argv[0]);
+					exit(EXIT_SUCCESS);
 				case 'a':
 					ufd_c++;
 					break;
 				case 'V':
-					do_version();
+					print_version();
 					exit(EXIT_SUCCESS);
-					break;
 			}
 		}
 
@@ -588,6 +605,7 @@ void init(int argc, char * argv[]) {
 					if (optarg[0] == '\0') {
 						fprintf(stderr, "%s: argument %i is a bit too short.\n", argv[0], optind - 1);
 						print_usage(argv[0]);
+						exit(EXIT_FAILURE);
 					}
 					break;
 			}
@@ -610,6 +628,7 @@ void init(int argc, char * argv[]) {
 					if (strlen(optarg) >= IFNAMSIZ) {
 						fprintf(stderr, "%s: argument %i is too long for a valid interface name.\n", argv[0], optind - 1);
 						print_usage(argv[0]);
+						exit(EXIT_FAILURE);
 					}
 					strncpy(public_ifname, optarg, IFNAMSIZ);
 					break;
@@ -617,6 +636,7 @@ void init(int argc, char * argv[]) {
 					if (inet_aton(optarg, &laddresses[i]) == 0) {
 						fprintf(stderr, "%s: argument %i is not a valid ip address.\n", argv[0], optind - 1);
 						print_usage(argv[0]);
+						exit(EXIT_FAILURE);
 					}
 					i++;
 					break;
@@ -625,6 +645,7 @@ void init(int argc, char * argv[]) {
 					if (max_lifetime == 0) {
 						fprintf(stderr, "%s: argument %i is not a valid lifetime.\n", argv[0], optind - 1);
 						print_usage(argv[0]);
+						exit(EXIT_FAILURE);
 					}
 					break;
 				case 'l': /* lowest port number allowed */
@@ -632,6 +653,7 @@ void init(int argc, char * argv[]) {
 					if (port_range_low == 0) {
 						fprintf(stderr, "%s: argument %i is not a valid port number.\n", argv[0], optind - 1);
 						print_usage(argv[0]);
+						exit(EXIT_FAILURE);
 					}
 					break;
 				case 'u': /* highest port number allowed */
@@ -639,22 +661,26 @@ void init(int argc, char * argv[]) {
 					if (port_range_high == 0) {
 						fprintf(stderr, "%s: argument %i is not a valid port number.\n", argv[0], optind - 1);
 						print_usage(argv[0]);
+						exit(EXIT_FAILURE);
 					}
 					break;
 				default: /* invalid option */
 					fprintf(stderr, "%s: argument %i is invalid.\n", argv[0], optind - 1);
 					print_usage(argv[0]);
+					exit(EXIT_FAILURE);
 			}
 		}
 
 		if (public_ifname[0] == 0) {
 			fprintf(stderr, "%s: option required -- i\n", argv[0]);
 			print_usage(argv[0]);
+			exit(EXIT_FAILURE);
 		}
 
 		if (ufd_c == 0) {
 			fprintf(stderr, "%s: option required -- a\n", argv[0]);
 			print_usage(argv[0]);
+			exit(EXIT_FAILURE);
 		}
 
 		/* port_range_low <= port_range_high; else bahaviour is undefined
@@ -662,6 +688,7 @@ void init(int argc, char * argv[]) {
 		if (port_range_high < port_range_low) {
 			fprintf(stderr, "%s: lower port may not be smaller than upper port. \n", argv[0]);
 			print_usage(argv[0]);
+			exit(EXIT_FAILURE);
 		}
 
 		/* port_low_offset >= port_range_low; else range not guaranteed */
